@@ -23,29 +23,59 @@
  *     FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *     General Public License for more details.
  * 
- *     You should have received a copy of the GNU General
- *     Public License along with this program; if not,
- *     write to the Free Software Foundation, Inc., 59 Temple 
- *     Place - Suite 330, Boston, MA  02111, USA.
+ *     You should have received a copy of the GNU General Public License
+ *     along with this program; if not, write to the Free Software Foundation,
+ *     Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,, USA.
  */
 
-/* $Id: helpers.h,v 1.2 2000/05/01 09:31:06 roland Exp $ */
-
 #ifndef _HELPERS_H
 #define _HELPERS_H
 
 #include <ctype.h>
+#include <sys/types.h>
+#include <string.h>
+#include "rfc822.h"
 
 char *safe_strdup (const char *);
 void *safe_calloc (size_t, size_t);
 void *safe_malloc (unsigned int);
 void safe_realloc (void **, size_t);
-void safe_free (void **);
+void safe_free (void *);
+size_t safe_strlen (const char *);
+int safe_strcmp (const char *, const char *);
 
-#define FREE(x) safe_free((void **)x)
+#define FREE(x) safe_free(x)
 #define SKIPWS(c) while (*(c) && isspace ((unsigned char) *(c))) c++;
-#define strfcpy(A,B,C) strncpy(A,B,C), *(A+(C)-1)=0
+#define strfcpy(A,B,C) strncpy(A,B,C), *((A)+(C)-1)=0
+# define NONULL(x) x?x:""
 
-#define STRING 256
+#define STRING 		256
+#define LONG_STRING    1024
+
+#define EMAIL_WSP " \t\r\n"
+
+/* skip over WSP as defined by RFC5322.  This is used primarily for parsing
+ * header fields. */
+
+static inline char *skip_email_wsp(const char *s)
+{
+  if (s)
+    return (char *)(s + strspn(s, EMAIL_WSP));
+  return (char *)s;
+}
+
+static inline int is_email_wsp(char c)
+{
+  return c && (strchr(EMAIL_WSP, c) != NULL);
+}
+
+#define ascii_strcmp(a,b) safe_strcmp(a,b)
+int ascii_strcasecmp (const char *a, const char *b);
+
+
+static inline const char *mutt_addr_for_display (ADDRESS *a)
+{
+  return a->mailbox;
+}
 
 #endif
